@@ -8,6 +8,6 @@ well when you're using setup.py to install e.g. paramiko!
 """
 from fabric.context_managers import cd, hide, settings, show
 from fabric.decorators import hosts, roles, runs_once
-from fabric.operations import require, prompt, put, get, run, sudo, local
-from fabric.state import env
-from fabric.utils import abort, warn
+from fabric.operations import require, prompt, put, get, run, sudo, local, reboot
+from fabric.state import env, output
+from fabric.utils import abort, warn, puts, fastprint
