@@ -14,8 +14,11 @@ from fabric.version import get_version
 # Win32 flag
 #
 
-# Impacts a handful of platform specific behaviors.
-win32 = sys.platform in ['win32', 'cygwin']
+# Impacts a handful of platform specific behaviors. Note that Cygwin's Python
+# is actually close enough to "real" UNIXes that it doesn't need (or want!) to
+# use PyWin32 -- so we only test for literal Win32 setups (vanilla Python,
+# ActiveState etc) here.
+win32 = (sys.platform == 'win32')
 
 
 #
@@ -163,7 +166,7 @@ env_options = [
     ),
 
     make_option('-f', '--fabfile',
-        default='fabfile.py',
+        default='fabfile',
         help="Python module file to import, e.g. '../other.py'"
     ),
 
@@ -208,11 +211,12 @@ env_options = [
 # preserving DRY: anything in here is generally not settable via the command
 # line.
 env = _AttributeDict({
-    'all_hosts': None, 
+    'all_hosts': [],
     'command': None,
     'cwd': '', # Must be empty string, not None, for concatenation purposes
     'host': None,
     'host_string': None,
+    'local_user': _get_system_username(),
     'port': None,
     'real_fabfile': None,
     'roledefs': {},
@@ -318,9 +322,9 @@ output = _AliasDict({
     'running': True,
     'stdout': True,
     'stderr': True,
-    'debug': False
-
+    'debug': False,
+    'user': True
 }, aliases={
-    'everything': ['warnings', 'running', 'output'],
+    'everything': ['warnings', 'running', 'user', 'output'],
     'output': ['stdout', 'stderr']
 })
